@@ -12,6 +12,7 @@ Options:
   -v, --version  Print module version and exit.
   --branch=<b>   Branch or tag [default: HEAD] up to which to check.
   --sort=<key>   [default: loc]|commits|files|hours|months.
+  --min=<val>    Minimum value (of `--sort` key) to show [default: 0:int].
   --loc=<type>   surv(iving)|ins(ertions)|del(etions)
                  What `loc` represents. Use 'ins,del' to count both.
                  defaults to 'surviving' unless `--cost` is specified.
@@ -50,19 +51,17 @@ Options:
   --ignore-revs-file=<f>   Ignore revisions listed in the given file
                            (requires `--loc=surviving`).
   --format=<format>        Table format
-      [default: pipe]|md|markdown|yaml|yml|json|csv|tsv|tabulate.
+      svg|[default: pipe]|md|markdown|yaml|yml|json|csv|tsv|tabulate.
       May require `git-fame[<format>]`, e.g. `pip install git-fame[yaml]`.
       Any `tabulate.tabulate_formats` is also accepted.
   --manpath=<path>         Directory in which to install git-fame man pages.
   --log=<lvl>    FATAL|CRITICAL|ERROR|WARN(ING)|[default: INFO]|DEBUG|NOTSET.
 """
-from __future__ import division, print_function
-
 import logging
 import os
 import re
 import subprocess
-# from __future__ import absolute_import
+from collections import defaultdict
 from functools import partial
 from os import path
 
@@ -79,7 +78,7 @@ except ImportError:
     except (ImportError, LookupError):
         __version__ = "UNKNOWN"
 __author__ = "Casper da Costa-Luis <casper.dcl@physics.org>"
-__date__ = "2016-2023"
+__date__ = "2016-2025"
 __licence__ = "[MPLv2.0](https://mozilla.org/MPL/2.0/)"
 __all__ = ["main"]
 __copyright__ = ' '.join(("Copyright (c)", __date__, __author__, __licence__))
@@ -120,28 +119,27 @@ def hours(dates, maxCommitDiffInSec=120 * 60, firstCommitAdditionInMinutes=120):
     """
     dates = sorted(dates)
     diffInSec = [i - j for (i, j) in zip(dates[1:], dates[:-1])]
-    res = sum(filter(lambda i: i < maxCommitDiffInSec, diffInSec))
+    res = sum(i for i in diffInSec if i < maxCommitDiffInSec)
     return (res/60.0 + firstCommitAdditionInMinutes) / 60.0
 
 
 def tabulate(auth_stats, stats_tot, sort='loc', bytype=False, backend='md', cost=None,
-             row_nums=False):
+             row_nums=False, min_sort_val=0, width=TERM_WIDTH):
     """
     backends  : [default: md]|yaml|json|csv|tsv|tabulate|
       `in tabulate.tabulate_formats`
     """
     COL_NAMES = ['Author', 'loc', 'coms', 'fils', ' distribution']
-    it_as = getattr(auth_stats, 'iteritems', auth_stats.items)
     # get ready
     tab = [[
         auth, s['loc'],
         s.get('commits', 0),
         len(s.get('files', [])), '/'.join(
-            map('{0:4.1f}'.format,
+            map('{:4.1f}'.format,
                 (100 * s['loc'] / max(1, stats_tot['loc']),
                  100 * s.get('commits', 0) / max(1, stats_tot['commits']),
                  100 * len(s.get('files', [])) / max(1, stats_tot['files'])))).replace(
-                     '/100.0/', '/ 100/')] for (auth, s) in it_as()]
+                     '/100.0/', '/ 100/')] for (auth, s) in auth_stats.items()]
     if cost:
         stats_tot = dict(stats_tot)
         if cost & COST_MONTHS:
@@ -155,8 +153,10 @@ def tabulate(auth_stats, stats_tot, sort='loc', bytype=False, backend='md', cost
         stats_tot.setdefault('hours', '%.1f' % sum(i[1] for i in tab))
     # log.debug(auth_stats)
 
-    for i, j in [("commits", "coms"), ("files", "fils"), ("hours", "hrs"), ("months", "mths")]:
+    for i, j in (("commits", "coms"), ("files", "fils"), ("hours", "hrs"), ("months", "mths")):
         sort = sort.replace(i, j)
+    if min_sort_val:
+        tab = [i for i in tab if i[COL_NAMES.index(sort)] >= min_sort_val]
     tab.sort(key=lambda i: i[COL_NAMES.index(sort)], reverse=True)
     if row_nums:
         tab = [[str(i)] + j for i, j in enumerate(tab, 1)]
@@ -164,16 +164,18 @@ def tabulate(auth_stats, stats_tot, sort='loc', bytype=False, backend='md', cost
 
     totals = 'Total ' + '\nTotal '.join("%s: %s" % i for i in sorted(stats_tot.items())) + '\n'
 
-    backend = backend.lower()
-    if backend in ("tabulate", "md", "markdown"):
+    if (backend := backend.lower()) in ("tabulate", "md", "markdown"):
         backend = "pipe"
+    svg = backend == 'svg'
+    if svg:
+        backend = 'rounded_outline'
 
-    if backend in ['yaml', 'yml', 'json', 'csv', 'tsv']:
+    if backend in ('yaml', 'yml', 'json', 'csv', 'tsv'):
         tab = [i[:-1] + [float(pc.strip()) for pc in i[-1].split('/')] for i in tab]
         tab = {
             "total": stats_tot, "data": tab,
             "columns": COL_NAMES[:-1] + ['%' + i for i in COL_NAMES[-4:-1]]}
-        if backend in ['yaml', 'yml']:
+        if backend in ('yaml', 'yml'):
             log.debug("backend:yaml")
             from yaml import safe_dump as tabber
             return tabber(tab).rstrip()
@@ -181,11 +183,11 @@ def tabulate(auth_stats, stats_tot, sort='loc', bytype=False, backend='md', cost
             log.debug("backend:json")
             from json import dumps as tabber
             return tabber(tab, ensure_ascii=False)
-        elif backend in ['csv', 'tsv']:
+        elif backend in ('csv', 'tsv'):
             log.debug("backend:csv")
             from csv import writer as tabber
+            from io import StringIO
 
-            from ._utils import StringIO
             res = StringIO()
             t = tabber(res, delimiter=',' if backend == 'csv' else '\t')
             t.writerow(tab['columns'])
@@ -198,13 +200,25 @@ def tabulate(auth_stats, stats_tot, sort='loc', bytype=False, backend='md', cost
             raise RuntimeError("Should be unreachable")
     else:
         import tabulate as tabber
+
         if backend not in tabber.tabulate_formats:
             raise ValueError(f"Unknown backend:{backend}")
         log.debug("backend:tabulate:%s", backend)
         COL_LENS = [max(len(Str(i[j])) for i in [COL_NAMES] + tab) for j in range(len(COL_NAMES))]
-        COL_LENS[0] = min(TERM_WIDTH - sum(COL_LENS[1:]) - len(COL_LENS) * 3 - 4, COL_LENS[0])
+        COL_LENS[0] = min(width - sum(COL_LENS[1:]) - len(COL_LENS) * 3 - 4, COL_LENS[0])
         tab = [[i[0][:COL_LENS[0]]] + i[1:] for i in tab]
-        return totals + tabber.tabulate(tab, COL_NAMES, tablefmt=backend, floatfmt='.0f')
+        table = tabber.tabulate(tab, COL_NAMES, tablefmt=backend, floatfmt='.0f')
+        if svg:
+            rows = table.split('\n')
+            return ('<svg xmlns="http://www.w3.org/2000/svg"'
+                    f' width="{len(rows[0]) / 2 + 1}em" height="{len(rows)}em">'
+                    '<rect x="0" y="0" width="100%" height="100%"'
+                    ' fill="white" fill-opacity="0.5" rx="5"/>'
+                    '<text x="0" y="-0.5em" font-size="15"'
+                    ' font-family="monospace" style="white-space: pre">' +
+                    ''.join(f'<tspan x="0" dy="1em">{row}</tspan>'
+                            for row in rows) + '</text></svg>')
+        return totals + table
 
         # from ._utils import tighten
         # return totals + tighten(tabber(...), max_width=TERM_WIDTH)
@@ -257,23 +271,16 @@ def _get_auth_stats(gitdir, branch="HEAD", since=None, include_files=None, exclu
     auth_stats = {}
 
     def stats_append(fname, auth, loc, tstamp):
-        auth = str(auth)
         tstamp = int(tstamp)
-        try:
-            auth_stats[auth]["loc"] += loc
-        except KeyError:
-            auth_stats[auth] = {"loc": loc, "files": {fname}, "ctimes": []}
-        else:
-            auth_stats[auth]["files"].add(fname)
-            auth_stats[auth]["ctimes"].append(tstamp)
+        if (auth := str(auth)) not in auth_stats:
+            auth_stats[auth] = defaultdict(int, files=set(), ctimes=[])
+        auth_stats[auth]["loc"] += loc
+        auth_stats[auth]["files"].add(fname)
+        auth_stats[auth]["ctimes"].append(tstamp)
 
         if bytype:
             fext_key = f".{fext(fname) or '_None_ext'}"
-            # auth_stats[auth].setdefault(fext_key, 0)
-            try:
-                auth_stats[auth][fext_key] += loc
-            except KeyError:
-                auth_stats[auth][fext_key] = loc
+            auth_stats[auth][fext_key] += loc
 
     if churn & CHURN_SLOC:
         for fname in tqdm(file_list, desc=gitdir if prefix_gitdir else "Processing",
@@ -325,16 +332,14 @@ def _get_auth_stats(gitdir, branch="HEAD", since=None, include_files=None, exclu
                 except ValueError:
                     log.warning(i)
                 else:
-                    fname = RE_RENAME.sub(r'\\2', fname)
-                    loc = (int(inss) if churn & CHURN_INS and inss else
-                           0) + (int(dels) if churn & CHURN_DEL and dels else 0)
-                    stats_append(fname, auth, loc, tstamp)
+                    if (fname := RE_RENAME.sub(r'\\2', fname)) in file_list:
+                        loc = int(inss) if churn & CHURN_INS and inss else 0
+                        loc += int(dels) if churn & CHURN_DEL and dels else 0
+                        stats_append(fname, auth, loc, tstamp)
 
     # quickly count commits (even if no surviving loc)
     log.log(logging.NOTSET, "authors:%s", list(auth_stats.keys()))
     auth_commits = check_output(git_cmd + ["shortlog", "-s", "-e", branch] + since + until)
-    for stats in auth_stats.values():
-        stats.setdefault("commits", 0)
     log.debug(RE_NCOM_AUTH_EM.findall(auth_commits.strip()))
     auth2em = {}
     auth2name = {}
@@ -342,23 +347,21 @@ def _get_auth_stats(gitdir, branch="HEAD", since=None, include_files=None, exclu
         auth = f'{name} <{em}>'
         auth2em[auth] = em
         auth2name[auth] = name
-        try:
-            auth_stats[auth]["commits"] += int(ncom)
-        except KeyError:
-            auth_stats[auth] = {"loc": 0, "files": set(), "commits": int(ncom), "ctimes": []}
+        if auth not in auth_stats:
+            auth_stats[auth] = defaultdict(int, files=set(), ctimes=[])
+        auth_stats[auth]["commits"] += int(ncom)
     if not (show & SHOW_NAME and show & SHOW_EMAIL): # replace author with either email or name
         auth2new = auth2em if (show & SHOW_EMAIL) else auth2name
         log.debug(auth2new)
         old = auth_stats
         auth_stats = {}
 
-        for auth, stats in getattr(old, 'iteritems', old.items)():
-            i = auth_stats.setdefault(auth2new[auth],
-                                      {"loc": 0, "files": set(), "commits": 0, "ctimes": []})
-            i["loc"] += stats["loc"]
+        for auth, stats in old.items():
+            i = auth_stats.setdefault(auth2new[auth], defaultdict(int, files=set(), ctimes=[]))
             i["files"].update(stats["files"])
-            i["commits"] += stats["commits"]
-            i["ctimes"] += stats["ctimes"]
+            for k, v in stats.items():
+                if k != 'files':
+                    i[k] += v
         del old
 
     return auth_stats
@@ -462,7 +465,7 @@ def run(args):
         mapper = map
 
     for res in mapper(statter, gitdirs):
-        for auth, stats in getattr(res, 'iteritems', res.items)():
+        for auth, stats in res.items():
             if auth in auth_stats:
                 merge_stats(auth_stats[auth], stats)
             else:
@@ -482,7 +485,8 @@ def run(args):
     # log.debug(extns)
 
     print_unicode(
-        tabulate(auth_stats, stats_tot, args.sort, args.bytype, args.format, cost, args.enum))
+        tabulate(auth_stats, stats_tot, args.sort, args.bytype, args.format, cost, args.enum,
+                 args.min))
 
 
 def get_main_parser():
